# Solution to exercise 2.4
from numpy import *
from matplotlib import pylab as plt

# Domain
[X0,X1] = plt.meshgrid(linspace(-3.,3.,1000),linspace(-3.,3.,1000))

# Function
F = exp(-X0**2-X1**2)*sin(4*(X0+X1+X0*X1**2))

# Plot the function
plt.clf()
plt.contour(X0,X1,F)
plt.colorbar()
plt.jet()
plt.xlabel("x0")
plt.ylabel("x1")
plt.ylim([-3,3])
plt.xlim([-3,3])


# solve with different starting points
from casadi import *
x = SX.sym('x',2)
f = exp(-x[0]**2 - x[1]**2 ) * sin(4*(x[0] + x[1] + x[0]*x[1]**2 ))
nlp = SXFunction( nlpIn(x=x), nlpOut(f=f) )
solver = NlpSolver('ipopt', nlp)
solver.setOption('tol', 1e-10)
solver.init()

# Gradient of f
gf = SXFunction([x], [gradient(f,x)])
gf.init()

# Solve for three different starting points
summary = [("GUESS", "SOLUTION", "SOLVER STATUS")]
for x_guess in [[0, 0], [0.9, 0.9], [-0.9, -0.9]]:
    # Pass as initial guess to the NLP solver
    solver.setInput(x_guess, 'x0')
    
    # Solve the NLP and get output
    solver.evaluate()
    x_opt = solver.getOutput('x')
    summary.append((x_guess, x_opt, solver.getStat("return_status")))
    plt.plot([x_guess[0],x_opt[0]], [x_guess[1],x_opt[1]],'ro-')

print "SUMMARY: "
for (x0,x,status) in summary: 
    print "%20s  %20s  %20s" % (x0,x,status)
    if not isinstance(x0, basestring):
      # Check optimality
      [g] = gf([x])
      print "  Gradient:   %20s" % g

      # Calculate the factors
      factor1 = exp(-x[0]**2-x[1]**2)
      factor2 = sin(4*(x[0]+x[1]+x[0]*x[1]**2))
      print "  Factors: %20s  %20s" % (factor1,factor2)


# Show plot
plt.axis([-3,3,-3,3])
plt.show()
